@@ -316,16 +316,21 @@ class LiveCDCopyBackend(backend.AnacondaBackend):
             "splash", "console=", "pci=routeirq", "irqpoll", "nohdparm", "pci=",
             "floppy.floppy=", "all-generic-ide", "gentoo=", "res=", "hsync=",
             "refresh=", "noddc", "xdriver=", "onlyvesa", "nvidia=", "dodmraid",
-            "dmraid", "sabayonmce", "quiet", "scandelay=", "doslowusb",
-            "docrypt", "dokeymap", "keymap=", "radeon.modeset=", "modeset=",
-            "nomodeset", "domdadm"]
+            "dmraid", "sabayonmce", "steambox", "quiet", "scandelay=",
+            "doslowusb", "docrypt", "dokeymap", "keymap=", "radeon.modeset=",
+            "modeset=", "nomodeset", "domdadm"]
 
-        # Sabayon MCE install -> MCE support
         # use reference, yeah
         cmdline = self._sabayon_install.cmdline
+
+        # Sabayon MCE install -> MCE support
         if Entropy.is_sabayon_mce() and ("sabayonmce" not in cmdline):
             cmdline.append("sabayonmce")
 
+        # Sabayon Steam Box support
+        if Entropy.is_sabayon_steambox() and ("steambox" not in cmdline):
+            cmdline.append("steambox")
+
         # Setup genkernel (init) keyboard layout
         if gk_kbd is not None:
             if "dokeymap" not in cmdline:
